// copyright ############################### //
// This file is part of the Xtrack Package.  //
// Copyright (c) CERN, 2025.                 //
// ######################################### //
#ifndef XTRACK_TRACK_SROTATION_H
#define XTRACK_TRACK_SROTATION_H

#include <headers/track.h>


GPUFUN
void SRotation_single_particle(LocalParticle* part, double sin_z, double cos_z)
{
    double const x  = LocalParticle_get_x(part);
    double const y  = LocalParticle_get_y(part);
    double const px = LocalParticle_get_px(part);
    double const py = LocalParticle_get_py(part);

    double const x_hat  =  cos_z * x  + sin_z * y;
    double const y_hat  = -sin_z * x  + cos_z * y;
    double const px_hat =  cos_z * px + sin_z * py;
    double const py_hat = -sin_z * px + cos_z * py;

    /* Spin tracking is disabled by the synrad compile flag */
    #ifndef XTRACK_MULTIPOLE_NO_SYNRAD
        // Rotate spin
        double const spin_x_0 = LocalParticle_get_spin_x(part);
        double const spin_y_0 = LocalParticle_get_spin_y(part);
        if ((spin_x_0 != 0) || (spin_y_0 != 0)){
            double const spin_x_1 = cos_z*spin_x_0 + sin_z*spin_y_0;
            double const spin_y_1 = -sin_z*spin_x_0 + cos_z*spin_y_0;
            LocalParticle_set_spin_x(part, spin_x_1);
            LocalParticle_set_spin_y(part, spin_y_1);
        }
    #endif

    LocalParticle_set_x(part, x_hat);
    LocalParticle_set_y(part, y_hat);
    LocalParticle_set_px(part, px_hat);
    LocalParticle_set_py(part, py_hat);
}

#endif